@@ -1,5 +1,3 @@
-from typing import Any, Dict, List, Optional, Tuple, Union
-
 from prophet import Prophet
 
 from prophet.diagnostics import cross_validation
@@ -9,25 +7,30 @@ from prophet.utilities import regressor_coefficients
 from tabulate import tabulate
 from sklearn.metrics import r2_score, mean_absolute_error
 
-from tscausalinference.evaluators import mape
-
 import pandas as pd
 from pandas import DataFrame
 
+import numpy as np
+
 import logging
 
+from tscausalinference.evaluators import mape
+
 logger = logging.getLogger('cmdstanpy')
 logger.addHandler(logging.NullHandler())
 logger.propagate = False
 logger.setLevel(logging.CRITICAL)
 
+pd.options.mode.chained_assignment = None 
+
 def synth_analysis(df: DataFrame = None, 
                     regressors: list = [], 
                     intervention: list = None, 
                     seasonality: bool = True,
-                    cross_validation_steps: int = 5
+                    cross_validation_steps: int = 5,
+                    alpha: float = 0.05,
+                    model_mode: str = 'additive'
                     ):
-
     """
     A function to analyze a dataset using the Prophet library and provide evaluation metrics, response metrics and 
     intervention metrics. 
@@ -81,27 +84,6 @@ def synth_analysis(df: DataFrame = None,
     | regressor_2   |         -0.0133  |
     +---------------+------------------+
 
-    Pre-Intervention response metrics ->
-    +---------+-------+
-    | Metric  | Value |
-    +=========+=======+
-    | r2      |  0.73 |
-    +---------+-------+
-    | MAE     | 34.92 |
-    +---------+-------+
-    | MAPE (%)| 15.21 |
-    +---------+-------+
-
-    Intervention metrics ->
-    +------------------------+--------+
-    | Metric                 | Value  |
-    +========================+========+
-    | Actual cumulative      | 156874 |
-    +------------------------+--------+
-    | Predicted cumulative:  | 163998 |
-    +------------------------+--------+
-    | Difference             |  7124  |
-    +------------------------+--------+
     """
 
     if not isinstance(df, pd.DataFrame):
@@ -128,15 +110,21 @@ def synth_analysis(df: DataFrame = None,
 
     print('Training period: {} to {}'.format(pre_intervention[0], pre_intervention[1]))
     print('Test period: {} to {}\n'.format(intervention[0], intervention[1]))
+    print('Prediction horizon: {} days'.format(prediction_period))
 
     if seasonality:
         prophet = Prophet(daily_seasonality=True, 
                     weekly_seasonality=True, 
                     changepoint_prior_scale = 0.05, 
-                    changepoint_range = 0.85)
+                    changepoint_range = 0.85,
+                    interval_width= 1 - alpha,
+                    seasonality_mode = model_mode)
     else:
-        prophet = Prophet(changepoint_prior_scale = 0.05, 
-                    changepoint_range = 0.85)
+        prophet = Prophet(
+                        changepoint_prior_scale = 0.05, 
+                        changepoint_range = 0.85,
+                        interval_width= 1 - alpha,
+                        seasonality_mode = model_mode)
 
     for regressor in regressors:
             prophet.add_regressor(name = regressor)
@@ -153,7 +141,7 @@ def synth_analysis(df: DataFrame = None,
     df['ds'] = pd.to_datetime(df['ds'])
 
     data = pd.merge(
-        prophet_predict[['ds','yhat', 'yhat_lower', 'yhat_upper']], 
+        prophet_predict[['ds','yhat', 'yhat_lower', 'yhat_upper', 'trend']+list(prophet.seasonalities.keys())], 
         df[["ds", "y"]], how='left', on='ds'
         )
 
@@ -170,7 +158,7 @@ def synth_analysis(df: DataFrame = None,
     data['cummulitive_yhat_upper'] = data['yhat_upper'].cumsum()
 
     # print response
-    print(f'Cross-validation MAPE: {model_mape_mean:.2%}')
+    print(f'\nCross-validation MAPE: {model_mape_mean:.2%}')
     if len(regressors) >= 1:
         regressor_df = regressor_coefficients(prophet)
         # format table
@@ -180,25 +168,34 @@ def synth_analysis(df: DataFrame = None,
 
         print(tabulate(table, headers=['Regressor', 'Coefficient'], tablefmt='grid'))
 
+    print('Seasons detected: {}'.format(list(prophet.seasonalities.keys())))
     pre_int_metrics = [
-    ['r2', r2_score(y_pred = data[(data.ds > pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat, y_true = data[(data.ds > pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y)],
-    ['MAE', mean_absolute_error(y_pred = data[(data.ds > pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat, y_true = data[(data.ds > pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y)],
-    ['MAPE (%)', mape(y_pred = data[(data.ds > pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat, y_true = data[(data.ds > pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y)]
+    ['r2', r2_score(y_pred = data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat, y_true = data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y)],
+    ['MAE', mean_absolute_error(y_pred = data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat, y_true = data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y)],
+    ['MAPE (%)', mape(y_pred = data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat, y_true = data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y)],
+    ['Noise (Std)', np.std(data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].y - data[(data.ds >= pd.to_datetime(pre_intervention[0]))&(data.ds <= pd.to_datetime(pre_intervention[1]))&(data.y > 0)].yhat)]
     ]
 
-    print('\nPre-Intervention response metrics ->')
-    print(tabulate(pre_int_metrics, headers=['Metric', 'Value'], tablefmt='grid'))
+    strings_info = """
++----------------------+
+Pre intervention metrics
++----------------------+
+{}
+    """
+
+    print(
+        strings_info.format(
+            tabulate(pre_int_metrics, 
+            headers=['Metric', 'Value'], 
+            tablefmt='pipe')
+        ).strip()
+    )
 
     int_metrics = [
-    ['Actual cumulative', data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].y.sum()],
-    ['Predicted cumulative:', data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].yhat.sum()],
-    ['Difference', data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].point_effects.sum()],
-    ['Change (%)', (data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].y.sum() / data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].yhat.sum() -1)*100]
+    ['Actual cumulative', data[(data.ds >= intervention[0]) & (data.ds <= intervention[1])].y.sum()],
+    ['Predicted cumulative:', data[(data.ds >= intervention[0]) & (data.ds <= intervention[1])].yhat.sum()],
+    ['Difference', data[(data.ds >= intervention[0]) & (data.ds <= intervention[1])].point_effects.sum()],
+    ['Change (%)', (data[(data.ds >= intervention[0]) & (data.ds <= intervention[1])].y.sum() / data[(data.ds >= intervention[0]) & (data.ds <= intervention[1])].yhat.sum() -1)*100]
     ]
-
-    print('\nIntervention response metrics ->')
-    print('Prediction Period {}'.format(prediction_period))
-    print('CI 95%: [{}, {}]'.format(data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].yhat_lower.sum(), data[(data.ds >= intervention[0]) & (data.ds < intervention[1])].yhat_upper.sum()))
-    print(tabulate(int_metrics, headers=['Metric', 'Value'], tablefmt='grid'))
-
-    return data
\ No newline at end of file
+    
+    return data, pre_int_metrics, int_metrics
\ No newline at end of file
